package dnd;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.DnDConstants;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.dnd.DragSourceAdapter;
import java.awt.dnd.DragSourceDropEvent;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetAdapter;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.TableModel;

import dnd.model.Part;
import dnd.model.PartTableModel;

public class DnD {

	// Erzeuge den DataFlavor fr "Part"-Objekte. Der magische
	// Teil mit "javaJVMLocalObjectMimeType" bewirkt, dass die
	// Objekte NICHT Serialiable sein mssen. (Symbol, z.B.,
	// ist nicht serialiable)
	private static DataFlavor createPartFlavor() {
		try {
			return new DataFlavor(DataFlavor.javaJVMLocalObjectMimeType + ";class=\"" + Part.class.getName() + "\"");
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			return null;
		}
	}

	// Eine Implementierung von Transferable, die direkt
	// ein "Part"-Objekt einwickelt
	private static class TransferablePart implements Transferable {

		private static final DataFlavor partFlavor = createPartFlavor();

		private final Part part;

		public TransferablePart(Part part) {
			this.part = part;
		}

		public DataFlavor[] getTransferDataFlavors() {
			return new DataFlavor[] { partFlavor };
		}

		public boolean isDataFlavorSupported(DataFlavor flavor) {
			return flavor.equals(partFlavor);
		}

		public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException {
			if (flavor.equals(partFlavor)) {
				return part;
			}
			throw new UnsupportedFlavorException(flavor);
		}
	}
	
	static void installDragHandling(JTable table) {
		
		// Wenn man auf der Table ein Drag-And-Drop anfngt, wird
		// das passende Part aus der Tabelle geholt, und das
		// passende TransferablePart erstellt
		DragGestureListener dragGestureListener = new DragGestureListener() {
			@Override
			public void dragGestureRecognized(DragGestureEvent dge) {
				if (table.getSelectedRowCount() != 1) {
					return;
				}
				int row = table.getSelectedRow();
				TableModel tableModel = table.getModel();
				PartTableModel<?> partTableModel = (PartTableModel<?>) tableModel;
				Part part = partTableModel.getPart(row);
				Transferable transferable = new TransferablePart(part);

				System.out.println("dragGestureRecognized with " + part);

				dge.startDrag(null, transferable);
			}
		};
		DragSource source = DragSource.getDefaultDragSource();
		source.createDefaultDragGestureRecognizer(table, DnDConstants.ACTION_COPY, dragGestureListener);
	}

	// Holt ein Part-Objekt aus einem TransferablePart raus
	private static Part extractPart(Transferable transferable) {
		try {
			Object result = transferable.getTransferData(TransferablePart.partFlavor);
			Part resultPart = (Part) result;
			return resultPart;
		} catch (UnsupportedFlavorException | IOException e) {
			e.printStackTrace();
			return null;
		}
	}

	// Ein Panel, das einen "Part" anzeigen kann. Das wird als Vorschau
	// verwendet, whrend man dragt: In dragEnter wird es erzeugt, in
	// dragOver wird es veschoben, und beim dragDropEnd wird es entfernt.
	// (Bei einem drop wird so eine Instanz auch erstellt, um das
	// gedroppte Part anzuzeigen...)
	static class PartPanel extends JPanel
	{
		PartPanel()
		{
			super(new BorderLayout(5, 0));
			setBorder(BorderFactory.createLineBorder(Color.BLACK));
		}
		void setPart(Part part)
		{
			removeAll();
			add(new JLabel(part.getIcon()), BorderLayout.WEST);
			add(new JLabel(part.getName()), BorderLayout.CENTER);
			add(new JLabel(part.getCode()), BorderLayout.EAST);
			setSize(getPreferredSize());
			doLayout();
		}
	}
	
	static void installDropHandling(JPanel dropPanel) {
		PartPanel draggedPartPanel = new PartPanel();
		
		DropTargetListener dropTargetListener = new DropTargetAdapter() {
			
			@Override
			public void drop(DropTargetDropEvent dtde) {
				
				Transferable transferable = dtde.getTransferable();
				Part part = extractPart(transferable);
				
				// Erzeuge ein neues PartPanel um das gedroppte
				// Part anzuzeigen
				PartPanel partPanel = new PartPanel();
				partPanel.setLocation(dtde.getLocation());
				partPanel.setPart(part);
				dropPanel.add(partPanel);
				dropPanel.repaint();
				
			}

			@Override
			public void dragOver(DropTargetDragEvent dtde) {
				
				System.out.println("dragOver to "+dtde.getLocation());

				// Aktualisiere die Position der Vorschau
				draggedPartPanel.setLocation(dtde.getLocation());
				dropPanel.repaint();
			}

			@Override
			public void dragExit(DropTargetEvent dte) {
				dropPanel.remove(draggedPartPanel);
			}

			@Override
			public void dragEnter(DropTargetDragEvent dtde) {
				
				Transferable transferable = dtde.getTransferable();
				Part part = extractPart(transferable);
				
				System.out.println("dragEnter with "+part);
				
				// Fge das Panel mit der Vorschau hinzu
				draggedPartPanel.setLocation(dtde.getLocation());
				draggedPartPanel.setPart(part);
				dropPanel.add(draggedPartPanel);
				dropPanel.repaint();
			}
		};
		new DropTarget(dropPanel, dropTargetListener);
		
		DragSource source = DragSource.getDefaultDragSource();
		source.addDragSourceListener(new DragSourceAdapter() {
			
			@Override
			public void dragDropEnd(DragSourceDropEvent dsde) {
				
				System.out.println("dragDropEnd");
				
				// Entferne die Vorschau (egal, ob das draggen
				// mit einem drop beendet oder abgebrochen wurde)
				dropPanel.remove(draggedPartPanel);
				dropPanel.repaint();
			}
		});
		
	}
}
